//
//  ViewController.m
//  FaustInstrumentIOS
//
//  Created by Romain Michon on 12/5/16.
//  Copyright © 2016 CCRMA. All rights reserved.
//

#import "ViewController.h"
#import "DspFaust.h"
#import <CoreMotion/CoreMotion.h>

// Accelerometer useful parameters
#define kMotionUpdateRate 30
#define ONE_G 9.81

@interface ViewController ()

@end

@implementation ViewController{
    DspFaust *dspFaust;
    CMMotionManager* _motionManager;
    NSTimer* _motionTimer;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    const int SR = 44100;
    const int bufferSize = 256;
    
    dspFaust = new DspFaust(SR,bufferSize);
    dspFaust->start();
    
    [self startMotion];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    
    [self stopMotion];
    dspFaust->stop();
    delete dspFaust;
}

- (void)startMotion
{
    if (_motionManager == nil)
    {
        _motionManager = [[CMMotionManager alloc] init];
        [_motionManager startAccelerometerUpdates];
        [_motionManager startGyroUpdates];
    }
    _motionTimer = [NSTimer scheduledTimerWithTimeInterval:1./kMotionUpdateRate target:self selector:@selector(updateMotion) userInfo:nil repeats:YES];
}

// Stop updating sensors
- (void)stopMotion
{
    if (_motionManager != nil)
    {
        [_motionManager stopAccelerometerUpdates];
        [_motionManager stopGyroUpdates];
        _motionManager = nil;
        [_motionTimer invalidate];
    }
}

- (void)updateMotion
{
    dspFaust->propagateAcc(0, _motionManager.accelerometerData.acceleration.x * ONE_G);
    dspFaust->propagateAcc(1, _motionManager.accelerometerData.acceleration.y * ONE_G);
    dspFaust->propagateAcc(2, _motionManager.accelerometerData.acceleration.z * ONE_G);
    dspFaust->propagateGyr(0, _motionManager.gyroData.rotationRate.x);
    dspFaust->propagateGyr(1, _motionManager.gyroData.rotationRate.y);
    dspFaust->propagateGyr(2, _motionManager.gyroData.rotationRate.z);
}

@end
